#include "tlang.h"

#if defined(USE_NETWORK)
    #include <errno.h>
    #if defined(GEKKO)
        #if defined(HW_RVL) && defined(USE_WIISOCKETS_ON_WII)
            #define USE_NORMAL_SOCKETS
        #else
            #define USE_NETWORK_H
            #include <network.h>
            #define read net_read
            #define write net_write
            #define connect net_connect
            #define listen net_listen
            #define bind net_bind
            #define close net_close
            #define accept net_accept
            #define socket net_socket
            char localip[16] = {0};
	        char gateway[16] = {0};
	        char netmask[16] = {0};

        #endif
    #else
        #define USE_NORMAL_SOCKETS
    #endif

    #if defined(USE_NORMAL_SOCKETS)
        #include <unistd.h>
        #include <arpa/inet.h>
        #include <sys/socket.h>
        #include <sys/select.h>
        #include <netdb.h>
        
    #endif
    bool networking_init()
    {
        #if defined(HW_RVL) && defined(USE_WIISOCKETS_ON_WII)
        return wiisocket_init() >= 0;
        #elif defined(USE_NETWORK_H)
        return if_config ( localip, netmask, gateway, TRUE, 20) >= 0;
        #else   
        
        #endif
        return true;
    }
    typedef struct 
    {
        int fd;
    } myfd_t;
    
    int64_t __fd_read(stream_t* strm,void* buff,int64_t len)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
       return (ssize_t) read(j->fd,buff,len);
    }
    void __fd_write(stream_t* strm,void* buff,int64_t len)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
       if(write(j->fd,buff,len) == -1)
       {
            printf("FAILED TO SEND\n");
       }
    }
    void __fd_close(stream_t* strm)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
        close(j->fd);
        free(j);
    }
     void __fd_flush(stream_t* strm)
    {
        myfd_t* j = (myfd_t*)strm->ptr;
        //flush(j->fd);
        
    }   
    stream_t* create_netfd_strm(int fd)
    {
        myfd_t* _fd = (myfd_t*)malloc(sizeof(myfd_t));
        _fd->fd = fd;
        return stream_create(_fd,__fd_read,__fd_write,NULL,NULL,NULL,stream_true,stream_true,stream_false,__fd_close,__fd_flush);    
    }
    
    
    stream_t* create_tcp_client(string_t* ipOrFqdn,int port)
    {
       int fd= socket(AF_INET,SOCK_STREAM,0);
       if(fd == -1)
       {return NULL;}
       struct sockaddr_in addr;
       memset(&addr,0,sizeof(struct sockaddr_in));
       char* ipAddress = string_dupp(ipOrFqdn);
        if (inet_aton(ipAddress,&addr.sin_addr) <=0)
        {
            Global_Mutex_Lock();
            struct hostent* host=gethostbyname(ipAddress);
            if(host != NULL)
            {
                memcpy(&addr.sin_addr,host->h_addr_list[0],sizeof(struct sockaddr_in));
            }
            Global_Mutex_Unlock();
        }
        free(ipAddress);
        addr.sin_port = htons((uint16_t)port);
        addr.sin_family = AF_INET;
        int res = connect(fd,&addr,sizeof(struct sockaddr_in));
        if(res != 0)
        {
            printf("ERROR AT NET: %s\n",strerror(errno));
            return NULL;
        }
        return create_netfd_strm(fd);
    }

    tobject_t* __net_tcpclient(runtime_t* rt,void* ptr,list_tobject_t* args)
    {
        if(args->length >1 && args->items[0]->type == tstring && args->items[1]->type == tnumber)
        {
            stream_t* strm = create_tcp_client(args->items[0]->data.string,(int)args->items[1]->data.number);
            if(strm == NULL)
            {
                return tobject_basic(rt,tnull);        
            }
            return tobject_fromstream(rt,strm);
        }
        return tobject_basic(rt,tnull);
    }
    
    void runtime_register_network(runtime_t* rt)
    {
        networking_init(); 
        tobject_t* __network = tobject_create(rt);
        __network->type = tdict;
        __network->data.dict = dict_create();
        runtime_create_method_on_dict(__network->data.dict,"tcpclient",rt,NULL,__net_tcpclient,NULL);
        rt->globals->setvariable(rt->globals,"net",__network);
        
    }

#endif